@@ -1,6 +1,6 @@
 /* simple-libmpd.c
  *
- * Copyright (c) 2006-2009 Landry Breuil (landry at fr.homeunix.org / gaston at gcu.info)
+ * Copyright (c) 2006-2010 Landry Breuil (landry at fr.homeunix.org / gaston at gcu.info)
  * This code is licenced under a BSD-style licence.
  * (OpenBSD variant modeled after the ISC licence)
  * All rights reserved.
@@ -291,6 +291,8 @@ void send_complex_cmd(MpdObj* mo, char* cmd, void (*parse_answer_fct)(), void *r
          {
             mo->error = MPD_ERROR_CONNCLOSED;
             DBG("ERROR parsing reply");
+            *mo->buffer = '\0';
+            mo->buflen = 0;
             return;
          }
 
@@ -412,7 +414,7 @@ void parse_playlistinfo_answer(MpdObj *mo, void *param)
       ms->id = ms->pos = -1;
       DBG("Going to parse song #%d", md->nb);
 
-      while(lines[i] && ms->id < 0)
+      while(lines[i] && strcmp(lines[i],"OK") && ms->id < 0)
       {
          tokens = g_strsplit(lines[i], ":", 2);
          /* remove leading whitespace */
@@ -434,6 +436,34 @@ void parse_playlistinfo_answer(MpdObj *mo, void *param)
    DBG("Got 'OK', md->nb = %d", md->nb);
 }
 
+void parse_outputs_answer(MpdObj *mo, void *param)
+{
+   MpdData* md = (MpdData*) param;
+   gchar **lines, **tokens;
+   int i = 0;
+   lines = g_strsplit(mo->buffer, "\n", 0);
+   while(lines[i] && strcmp(lines[i],"OK"))
+   {
+      md->alloutputs[md->nb] = g_new(mpd_Output, 1);
+      md->alloutputs[md->nb]->enabled = -1;
+      DBG("Going to parse output #%d", md->nb);
+      while(lines[i] && strcmp(lines[i],"OK") && md->alloutputs[md->nb]->enabled < 0)
+      {
+         tokens = g_strsplit(lines[i], ":", 2);
+         /* remove leading whitespace */
+         tokens[1] = g_strchug(tokens[1]);
+         DBG("key=\"%s\",value=\"%s\"", tokens[0], tokens[1]);
+         if      (0 == strcmp("outputid",tokens[0])) md->alloutputs[md->nb]->id = atoi(tokens[1]);
+         else if (0 == strcmp("outputname",tokens[0])) md->alloutputs[md->nb]->name = g_strdup(tokens[1]);
+         else if (0 == strcmp("outputenabled",tokens[0])) md->alloutputs[md->nb]->enabled = atoi(tokens[1]);
+         i++;
+         g_strfreev(tokens);
+      }
+      md->nb++;
+   }
+   g_strfreev(lines);
+}
+
 mpd_Song* mpd_playlist_get_current_song(MpdObj* mo)
 {
    DBG("!");
@@ -471,6 +501,7 @@ MpdData* mpd_playlist_get_changes(MpdObj* mo, int old_playlist_id)
 {
    MpdData* md = g_new0(MpdData,1);
    md->cur = md->nb = 0;
+   md->type = MPD_DATA_TYPE_SONG;
    md->allsongs = g_new(mpd_Song, mo->playlistlength);
    DBG("!");
    send_complex_cmd(mo, "playlistinfo\n", parse_playlistinfo_answer, (void*) md);
@@ -486,21 +517,63 @@ MpdData* mpd_data_get_next(MpdData* md)
    {
       for (md->cur--; md->cur; md->cur--)
       {
-         if (md->allsongs[md->cur].file) free(md->allsongs[md->cur].file);
-         if (md->allsongs[md->cur].artist) free(md->allsongs[md->cur].artist);
-         if (md->allsongs[md->cur].album) free(md->allsongs[md->cur].album);
-         if (md->allsongs[md->cur].title) free(md->allsongs[md->cur].title);
-         if (md->allsongs[md->cur].track) free(md->allsongs[md->cur].track);
+         switch (md->type) {
+            case MPD_DATA_TYPE_OUTPUT_DEV:
+            if (md->alloutputs[md->cur]->name) free(md->alloutputs[md->cur]->name);
+            break;
+            case MPD_DATA_TYPE_SONG:
+            if (md->allsongs[md->cur].file) free(md->allsongs[md->cur].file);
+            if (md->allsongs[md->cur].artist) free(md->allsongs[md->cur].artist);
+            if (md->allsongs[md->cur].album) free(md->allsongs[md->cur].album);
+            if (md->allsongs[md->cur].title) free(md->allsongs[md->cur].title);
+            if (md->allsongs[md->cur].track) free(md->allsongs[md->cur].track);
+            break;
+         }
+      }
+      switch (md->type) {
+         case MPD_DATA_TYPE_OUTPUT_DEV:
+            g_free(md->alloutputs);
+            break;
+         case MPD_DATA_TYPE_SONG:
+            g_free(md->allsongs);
+            break;
       }
-      g_free(md->allsongs);
       g_free(md);
       DBG("Free()'d md");
       return NULL;
    }
-   md->song = (&md->allsongs[md->cur]);
+   switch (md->type) {
+      case MPD_DATA_TYPE_OUTPUT_DEV:
+         md->output_dev = md->alloutputs[md->cur];
+         break;
+      case MPD_DATA_TYPE_SONG:
+         md->song = (&md->allsongs[md->cur]);
+         break;
+   }
    return md;
 }
 
+MpdData* mpd_server_get_output_devices(MpdObj* mo)
+{
+   MpdData* md = g_new0(MpdData,1);
+   DBG("!");
+   md->cur = md->nb = 0;
+   md->alloutputs = g_new(mpd_Output*,1);
+   md->type = MPD_DATA_TYPE_OUTPUT_DEV;
+   send_complex_cmd(mo, "outputs\n", parse_outputs_answer, (void*) md);
+   md->output_dev = md->alloutputs[0];
+   return ((!mo->error) ? md : NULL);
+}
+
+int mpd_server_set_output_device (MpdObj* mo, int id, int state)
+{
+   char outbuf[18];
+   /* write (enable|disable)output 'id' to socket */
+   DBG("!");
+   snprintf(outbuf, sizeof(outbuf), "%soutput %d\n",(state ? "enable" : "disable"), id);
+   return mpd_send_single_cmd(mo,outbuf);
+}
+
 void mpd_status_set_volume(MpdObj* mo, int newvol)
 {
    char outbuf[15];
